<?php
/**
 * Plugin Name:       PDF Embedder Premium
 * Plugin URI:        https://wp-pdf.com
 * Description:       Embed PDFs straight into your posts and pages, with flexible width and height. No third-party services required. Compatible with Gutenberg Editor WordPress
 * Requires Plugins:  pdf-embedder
 * Requires at least: 5.8
 * Requires PHP:      7.0
 * Version:           5.2.3
 * Author:            PDF Embedder
 * Author URI:        https://wp-pdf.com
 * Text Domain:       pdf-embedder-premium
 * Domain Path:       assets/languages
 *
 *  PDF Embedder is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  any later version.
 *
 *  PDF Embedder is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with PDF Embedder. If not, see <https://www.gnu.org/licenses/>.
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Plugin version.
 *
 * @since 5.2.0
 */
const PDFEMB_PREMIUM_VERSION = '5.2.3';

/**
 * Plugin Folder Path.
 *
 * @since 5.2.0
 */
define( 'PDFEMB_PREMIUM_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );

/**
 * Plugin Folder URL.
 *
 * @since 5.2.0
 */
define( 'PDFEMB_PREMIUM_PLUGIN_URL', plugin_dir_url( __FILE__ ) );

/**
 * Plugin Root File.
 *
 * @since 5.2.0
 */
const PDFEMB_PREMIUM_PLUGIN_FILE = __FILE__;

/**
 * Process plugin hard requirements.
 *
 * @since 5.2.0
 */
require_once __DIR__ . '/requirements.php';

// phpcs:ignore WPForms.PHP.BackSlash.UseShortSyntax
$pdfemb_premium_requirements = new \PDFEmbedder\Premium\Requirements();

if ( $pdfemb_premium_requirements->are_not_met() ) {

	$pdfemb_premium_requirements->notify();

	// Do not process the plugin code further.
	return;
}

require_once __DIR__ . '/vendor/autoload.php';

/*
 * Action Scheduler requires a special loading procedure.
 *
 * @since 5.2.0
 */
add_action(
	'plugins_loaded',
	static function() {
		require_once PDFEMB_PLUGIN_DIR . 'vendor/woocommerce/action-scheduler/action-scheduler.php';
	},
	-9
);

add_action(
	'pdfemb_loaded',
	static function ( $lite ) {

		// phpcs:ignore WPForms.PHP.BackSlash.UseShortSyntax
		$premium = new \PDFEmbedder\Premium\Plugin();

		$premium->init();

		$premium->hooks( $lite );
	}
);
