<?php

namespace PDFEmbedder\Premium\Plans\Available\Basic;

use PDFEmbedder\Premium\Plans\PremiumPlan;
use PDFEmbedder\Premium\Plans\Features\ExtendsAdminLicense;
use PDFEmbedder\Premium\Plans\Features\ExtendsOptionsWithExtraMobile;
use PDFEmbedder\Premium\Plans\Features\ExtendsAdminSettingsExtra;
use PDFEmbedder\Premium\Plans\Features\ExtendsAdminMobile;

/**
 * Basic plan class.
 *
 * @since 5.2.0
 */
class Basic extends PremiumPlan {
	use ExtendsOptionsWithExtraMobile;
	use ExtendsAdminSettingsExtra;
	use ExtendsAdminMobile;
	use ExtendsAdminLicense;

	/**
	 * Priority of the plan.
	 *
	 * @since 5.2.0
	 */
	const PRIORITY = 10;

	/**
	 * Name of the plan.
	 *
	 * @since 5.2.0
	 */
	const NAME = 'basic';

	/**
	 * Extend the functionality of WP and PDF Lite.
	 * All these hooks may be running from various plans,
	 * which may or may not have methods implemented - that's why we have __call() method.
	 *
	 * @since 5.2.0
	 */
	public function hooks() {

		parent::hooks();

		add_action( 'pdfemb_admin_settings_extra', [ $this, 'show_extra_settings' ], static::PRIORITY );
		add_action( 'pdfemb_admin_settings_render_section_mobile', [ $this, 'show_mobile_settings' ] );

		// New License tab.
		add_filter(
			'pdfemb_admin_sections',
			static function ( $sections ) {
				$sections['license'] = __( 'License', 'pdf-embedder-premium' );

				return $sections;
			}
		);
		add_action( 'pdfemb_admin_settings_render_section_license', [ $this, 'show_license_settings' ] );
	}
}
