<?php

namespace PDFEmbedder\Premium\Plans\Features;

use PDFEmbedder\Admin\License;
use PDFEmbedder\Helpers\Links;

/**
 * Extends the admin settings with the license tab.
 *
 * @since 5.2.0
 */
trait ExtendsAdminLicense {

	/**
	 * Render the License tab.
	 *
	 * @since 5.2.0
	 */
	public function show_license_settings() {

		$options = pdf_embedder()->options()->get();
		?>

		<h3 class="headline-title">
			<?php esc_html_e( 'License Information', 'pdf-embedder-premium' ); ?>
		</h3>

		<p><?php esc_html_e( 'You should have received a license key when you purchased this premium version of PDF Embedder.', 'pdf-embedder-premium' ); ?></p>
		<p>
			<?php
			printf(
				wp_kses( /* translators: %s - URL to contact the support. */
					__( 'Please enter it below to enable automatic updates, or <a href="%s" target="_blank">contact us</a> if you do not have one.', 'pdf-embedder-premium' ),
					[
						'a' => [
							'href'   => [],
							'target' => [],
						],
					]
				),
				esc_url( Links::get_utm_link( 'https://wp-pdf.com/contact/', 'Admin - License', 'contact us' ) )
			);
			?>
		</p>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-license_key">
			<label for="input_pdfemb_license_key" class="textinput big"><?php esc_html_e( 'License Key', 'pdf-embedder-premium' ); ?></label>
			<input id="input_pdfemb_license_key" name='pdfemb[pdfemb_license_key]' size='40' type='text' value='<?php echo esc_attr( $options['pdfemb_license_key'] ); ?>' class='textinput' />
		</div>

		<br class="clear" />

		<?php
		// Display latest license status.
		$license_status = get_site_option( License::OPTION_STATUS, [] );

		if ( is_array( $license_status ) && ! empty( $license_status['license_id'] ) ) {
			$date_format = get_option( 'date_format' ) . ' ' . get_option( 'time_format' );

			echo '<br class="clear" />';
			echo '<table>';
			if ( ! empty( $license_status['status'] ) ) {
				echo '<tr><td>' . esc_html__( 'Status:', 'pdf-embedder-premium' ) . '</td><td><code>' . esc_html( strtoupper( $license_status['status'] ) ) . '</code></td></tr>';
			}

			if ( ! empty( $license_status['last_check_time'] ) ) {
				echo '<tr><td>' . esc_html__( 'Last Checked:', 'pdf-embedder-premium' ) . '</td><td>' . esc_html( gmdate( $date_format, (int) $license_status['last_check_time'] ) ) . '</td></tr>';
			}

			if ( ! empty( $license_status['expires_time'] ) ) {
				?>
				<tr>
					<td><?php esc_html__( 'License Expires:', 'pdf-embedder-premium' ); ?></td>
					<td>
						<?php esc_html( gmdate( $date_format, (int) $license_status['expires_time'] ) ); ?>
						<?php
						if (
							$license_status['expires_time'] < time() + MONTH_IN_SECONDS &&
							! empty( $license_status['renewal_link'] )
						) {
							echo '<br />';
							printf(
								wp_kses( /* translators: %s - URL to renew the license. */
									__( '<a href="%s" target="_blank">Renew your license</a>.', 'pdf-embedder-premium' ),
									[
										'a' => [
											'href'   => [],
											'target' => [],
										],
									]
								),
								esc_url( $license_status['renewal_link'] )
							);
						}
						?>
					</td>
				</tr>
				<?php
			}

			echo '</table>';
		}
		?>

		<hr class="clear">

		<p class="submit">
			<button type="submit" class="button button-primary" id="submit" name="submit">
				<?php esc_html_e( 'Save Changes', 'pdf-embedder-premium' ); ?>
			</button>
		</p>

		<?php
	}
}
