<?php

namespace PDFEmbedder\Premium\Plans\Features;

use PDFEmbedder\Options;
use PDFEmbedder\Helpers\Links;

/**
 * Trait ExtendsAdminMobile.
 *
 * @since 5.2.0
 */
trait ExtendsAdminMobile {

	/**
	 * Register mobile plugin settings.
	 *
	 * @since 5.2.0
	 */
	public function show_mobile_settings() {

		$options = pdf_embedder()->options()->get();
		?>

		<h3><?php esc_html_e( 'Default Mobile Settings', 'pdf-embedder-premium' ); ?></h3>

		<p><?php esc_html_e( "When the document is smaller than the width specified below, the document displays only as a 'thumbnail' with a large 'View in Full Screen' button for the user to click to open.", 'pdf-embedder-premium' ); ?></p>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-mobilewidth">
			<label for="input_pdfemb_mobilewidth" class="textinput">
				<?php esc_html_e( 'Mobile Width', 'pdf-embedder-premium' ); ?>
			</label>

			<input id='input_pdfemb_mobilewidth' class='textinput' name='pdfemb[pdfemb_mobilewidth]' size='10' type='number' value='<?php echo esc_attr( $options['pdfemb_mobilewidth'] ); ?>' />

			<p class="desc clear">
				<?php esc_html_e( 'Enter an integer number of pixels, or 0 to disable automatic full-screen.', 'pdf-embedder-premium' ); ?>
			</p>
		</div>

		<br class="clear"/>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-mobilewidth_button_text">
			<label for="input_mobilewidth_button_text" class="textinput">
				<?php esc_html_e( 'Mobile Width Button Text', 'pdf-embedder-premium' ); ?>
			</label>

			<input id='input_mobilewidth_button_text' class='textinput' name='pdfemb[pdfemb_mobilewidth_button_text]' size='50' type='text' value='<?php echo esc_attr( $options['pdfemb_mobilewidth_button_text'] ); ?>' />

			<p class="desc clear">
				<?php esc_html_e( 'Enter a short string of text for the button.', 'pdf-embedder-premium' ); ?>
			</p>
		</div>

		<br class="clear"/>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-resetviewport">
			<label for="pdfemb_resetviewport" class="textinput">
				<?php esc_html_e( 'Disable Device Zoom', 'pdf-embedder-premium' ); ?>
			</label>

			<input type="checkbox" name='pdfemb[pdfemb_resetviewport]' id='pdfemb_resetviewport' class='checkbox' <?php echo Options::is_on( $options['pdfemb_resetviewport'] ) ? 'checked' : ''; ?> />
			<label for="pdfemb_resetviewport" class="checkbox plain"><?php esc_html_e( 'Enable if you are experiencing quality issues on mobile devices', 'pdf-embedder-premium' ); ?></label>

			<p class="desc clear">
				<?php esc_html_e( 'Some mobile browsers will use their own zoom, causing the PDF Embedder to render at a lower resolution than it should, or lose the toolbar off screen.', 'pdf-embedder-premium' ); ?>
				<br>
				<?php esc_html_e( 'Enabling this option may help, but could potentially affect appearance in the rest of your site.', 'pdf-embedder-premium' ); ?>
				<?php
				printf(
					wp_kses( /* translators: %s - link to documentation. */
						__( 'See <a href="%s" target="_blank">documentation</a> for details.', 'pdf-embedder-premium' ),
						[
							'a' => [
								'href'   => [],
								'target' => [],
							],
						]
					),
					esc_url( Links::get_utm_link( 'https://wp-pdf.com/premium-instructions/#disabledevicezoom', 'Admin - Mobile', 'Disable Device Zoom' ) )
				);
				?>
			</p>
		</div>

		<hr class="clear">

		<p class="submit">
			<button type="submit" class="button button-primary" id="submit" name="submit">
				<?php esc_html_e( 'Save Changes', 'pdf-embedder-premium' ); ?>
			</button>
		</p>

		<?php
	}
}
