<?php

namespace PDFEmbedder\Premium\Plans\Features;

use PDFEmbedder\Options;
use PDFEmbedder\Helpers\Links;

/**
 * Trait ExtendsAdminSecure.
 *
 * @since 5.2.0
 */
trait ExtendsAdminSecure {

	/**
	 * Register secure plugin settings.
	 *
	 * @since 5.2.0
	 */
	public function show_secure_settings() {

		$options = pdf_embedder()->options()->get();
		?>

		<h3><?php esc_html_e( 'Secure PDFs', 'pdf-embedder-premium' ); ?></h3>

		<p>
			<?php esc_html_e( 'Your server must be running Apache and supporting .htaccess rules to restrict access to files.', 'pdf-embedder-premium' ); ?>
		</p>
		<p>
			<?php
			printf(
				wp_kses( /* translators: %s - link to the documentation. */
					__( 'If your server is using nginx, you will need to manually configure it. <a href="%s" target="_blank">Read more in our docs</a>.', 'pdf-embedder-premium' ),
					[
						'a' => [
							'href'   => [],
							'target' => [],
						],
					]
				),
				esc_url( Links::get_utm_link( 'https://wp-pdf.com/kb/serving-protected-files-under-nginx/', 'Admin - Secure', 'nginx' ) )
			);
			?>
		</p>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-secure">
			<label for="pdfemb_secure" class="textinput"><?php esc_html_e( 'Secure PDFs', 'pdf-embedder-premium' ); ?></label>

			<span>
				<input type="checkbox" name='pdfemb[pdfemb_secure]' id='pdfemb_secure' class='checkbox' <?php checked( boolval( $options['pdfemb_secure'] ), 1 ); ?> />
				<label for="pdfemb_secure" class="checkbox plain"><?php esc_html_e( "Send new PDF media uploads to 'securepdfs' folder", 'pdf-embedder-premium' ); ?></label>
			</span>
		</div>

		<br class="clear" />

		<div class="pdfemb-admin-setting pdfemb-admin-setting-disablerightclick">
			<label for="pdfemb_disablerightclick" class="textinput"><?php esc_html_e( 'Disable Right Click', 'pdf-embedder-premium' ); ?></label>
			<span>
				<input type="checkbox" name='pdfemb[pdfemb_disablerightclick]' id='pdfemb_disablerightclick' class='checkbox' <?php checked( boolval( $options['pdfemb_disablerightclick'] ), 1 ); ?> />
				<label for="pdfemb_disablerightclick" class="checkbox plain"><?php esc_html_e( 'Disable right-click mouse menu (affects secure PDFs only)', 'pdf-embedder-premium' ); ?></label>
			</span>
		</div>

		<br class="clear" />

		<div class="pdfemb-admin-setting pdfemb-admin-setting-cacheencrypted">
			<label for="pdfemb_cacheencrypted" class="textinput"><?php esc_html_e( 'Cache Encrypted PDFs', 'pdf-embedder-premium' ); ?></label>
			<span>
				<input type="checkbox" name='pdfemb[pdfemb_cacheencrypted]' id='pdfemb_cacheencrypted' class='checkbox' <?php checked( boolval( $options['pdfemb_cacheencrypted'] ), 1 ); ?> />
				<label for="pdfemb_cacheencrypted" class="checkbox plain"><?php esc_html_e( 'Cache encrypted versions of secure PDFs on the server (increase download speed)', 'pdf-embedder-premium' ); ?></label>
			</span>
		</div>

		<br class="clear" />

		<div class="pdfemb-admin-setting pdfemb-admin-setting-secureattpages">
			<label for="pdfemb_secureattpages" class="textinput"><?php esc_html_e( 'Attachment Pages', 'pdf-embedder-premium' ); ?></label>
			<span>
				<input type="checkbox" name='pdfemb[pdfemb_secureattpages]' id='pdfemb_secureattpages' class='checkbox' <?php checked( boolval( $options['pdfemb_secureattpages'] ), 1 ); ?> />
				<label for="pdfemb_secureattpages" class="checkbox plain"><?php esc_html_e( 'Auto-generate Attachment Pages for Secure PDFs', 'pdf-embedder-premium' ); ?></label>
			</span>
		</div>

		<br class="clear" />
		<br class="clear" />

		<hr class="clear">

		<h3><?php esc_html_e( 'Watermark', 'pdf-embedder-premium' ); ?></h3>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-wm_text">
			<label for="pdfemb_wm_text" class="textinput"><?php esc_html_e( 'Text to display on secure PDFs', 'pdf-embedder-premium' ); ?></label>
			<textarea id='pdfemb_wm_text' class='textinput' name='pdfemb[pdfemb_wm_text]'><?php echo wp_kses_post( htmlentities( $options['pdfemb_wm_text'] ) ); ?></textarea>

			<p class="desc clear">

				<?php esc_html_e( 'Leave blank for no watermark on secure PDFs.', 'pdf-embedder-premium' ); ?><br />
				<?php
				printf(
					wp_kses( /* translators: %1$s, %2$s, %3$s - various smart tags. */
						__( 'You can use Smart Tags to display logged in user information: %1$s, %2$s, %3$s. For logged out users they will be blank.', 'pdf-embedder-premium' ),
						[
							'code' => [],
						]
					),
					'<code>{fullname}</code>',
					'<code>{username}</code>',
					'<code>{email}</code>'
				);
				?>
			</p>
		</div>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-wm_halign">
			<label for="pdfemb_wm_halign" class="textinput"><?php esc_html_e( 'Horizontal alignment', 'pdf-embedder-premium' ); ?></label>
			<select name='pdfemb[pdfemb_wm_halign]' id='pdfemb_wm_halign' class='select'>
				<option value="left" <?php echo $options['pdfemb_wm_halign'] === 'left' ? 'selected' : ''; ?>><?php esc_html_e( 'Left', 'pdf-embedder-premium' ); ?></option>
				<option value="center" <?php echo $options['pdfemb_wm_halign'] === 'center' ? 'selected' : ''; ?>><?php esc_html_e( 'Center', 'pdf-embedder-premium' ); ?></option>
				<option value="right" <?php echo $options['pdfemb_wm_halign'] === 'right' ? 'selected' : ''; ?>><?php esc_html_e( 'Right', 'pdf-embedder-premium' ); ?></option>
			</select>
		</div>

		<br class="clear"/>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-wm_voffset">
			<label for="pdfemb_wm_voffset" class="textinput"><?php esc_html_e( 'Vertical offset (%)', 'pdf-embedder-premium' ); ?></label>
			<span>
				<input id='pdfemb_wm_voffset' class='textinput' name='pdfemb[pdfemb_wm_voffset]' size='10' type='number' value='<?php echo esc_attr( $options['pdfemb_wm_voffset'] ); ?>' />
				<label for="pdfemb_wm_voffset" class="checkbox plain" style="margin: 21px 0 0 5px;"><?php esc_html_e( 'Numerical value between 0 and 100', 'pdf-embedder-premium' ); ?></label>
			</span>
		</div>

		<br class="clear"/>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-wm_fontsize">
			<label for="pdfemb_wm_fontsize" class="textinput"><?php esc_html_e( 'Font Size (pt)', 'pdf-embedder-premium' ); ?></label>
			<span>
				<input id='pdfemb_wm_fontsize' class='textinput' name='pdfemb[pdfemb_wm_fontsize]' size='10' type='number' value='<?php echo esc_attr( $options['pdfemb_wm_fontsize'] ); ?>' />
			</span>
		</div>

		<br class="clear"/>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-wm_opacity">
			<label for="pdfemb_wm_opacity" class="textinput"><?php esc_html_e( 'Opacity (%)', 'pdf-embedder-premium' ); ?></label>
			<span>
				<input id='pdfemb_wm_opacity' class='textinput' name='pdfemb[pdfemb_wm_opacity]' size='10' type='number' value='<?php echo esc_attr( $options['pdfemb_wm_opacity'] ); ?>' />
				<label for="pdfemb_wm_opacity" class="checkbox plain" style="margin: 21px 0 0 5px;"><?php esc_html_e( 'Numerical value between 0 and 100', 'pdf-embedder-premium' ); ?></label>
			</span>
		</div>

		<br class="clear"/>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-wm_rotate">
			<label for="pdfemb_wm_rotate" class="textinput"><?php esc_html_e( 'Rotation (degrees)', 'pdf-embedder-premium' ); ?></label>
			<span>
				<input id='pdfemb_wm_rotate' class='textinput' name='pdfemb[pdfemb_wm_rotate]' size='10' type='number' value='<?php echo esc_attr( $options['pdfemb_wm_rotate'] ); ?>' />
				<label for="pdfemb_wm_rotate" class="checkbox plain" style="margin: 21px 0 0 5px;"><?php esc_html_e( 'Numerical value in degrees (from 0 to 360)', 'pdf-embedder-premium' ); ?></label>
			</span>

			<p class="desc clear">
				<?php esc_html_e( 'If positive, the movement will be clockwise; if negative - counter-clockwise.', 'pdf-embedder-premium' ); ?>
			</p>
		</div>

		<br class="clear"/>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-wm_evenpages">
			<label for="pdfemb_wm_evenpages" class="textinput"><?php esc_html_e( 'Page Display', 'pdf-embedder-premium' ); ?></label>
			<span>
				<input type="checkbox" name='pdfemb[pdfemb_wm_evenpages]' id='pdfemb_wm_evenpages' class='checkbox' <?php echo Options::is_on( $options['pdfemb_wm_evenpages'] ) ? 'checked' : ''; ?> />
				<label for="pdfemb_wm_evenpages" class="checkbox plain"><?php esc_html_e( 'Show only on even page numbers', 'pdf-embedder-premium' ); ?></label>
			</span>
		</div>

		<br class="clear"/>

		<hr />

		<p>
			<?php
			esc_html_e(
				"If 'Secure PDFs' is checked above, your PDF uploads will be 'secure' by default.
               That is, they should be uploaded to a 'securepdfs' sub-folder of your uploads area. These files should not be accessible directly,
               and the plugin provides a backdoor method for the embedded viewer to obtain the file contents.",
				'pdf-embedder-premium'
			);
			?>
		</p>

		<p>
			<?php
			esc_html_e(
				'This means that your PDF is unlikely to be shared outside your site where you have no control over who views, prints, or shares it.
               Please note that it is still always possible for a determined user to obtain the original file. Sensitive information should never be presented to viewers in any form.',
				'pdf-embedder-premium'
			);
			?>
		</p>

		<p>
			<?php
			printf(
				wp_kses( /* translators: %s - link to instructions. */
					__( 'See <a href="%s" target="_blank">Instructions</a> for more details.', 'pdf-embedder-premium' ),
					[
						'a' => [
							'href'   => [],
							'target' => [],
						],
					]
				),
				esc_url( Links::get_utm_link( 'https://wp-pdf.com/secure-instructions/', 'Admin - Secure', 'instructions' ) )
			);
			?>
		</p>


		<hr class="clear">

		<p class="submit">
			<button type="submit" class="button button-primary" id="submit" name="submit">
				<?php esc_html_e( 'Save Changes', 'pdf-embedder-premium' ); ?>
			</button>
		</p>

		<?php
	}
}
