<?php

namespace PDFEmbedder\Premium\Plans\Features;

use PDFEmbedder\Options;

trait ExtendsAdminSettingsExtra {

	/**
	 * Register extra plugin settings.
	 *
	 * @since 5.2.0
	 */
	public function show_extra_settings() { // phpcs:ignore Generic.Metrics.CyclomaticComplexity.MaxExceeded

		$options = pdf_embedder()->options()->get();
		?>

		<br class="clear" />

		<div class="pdfemb-admin-setting pdfemb-admin-setting-scrollbar">
			<label for="pdfemb_scrollbar" class="textinput"><?php esc_html_e( 'Display Scrollbars', 'pdf-embedder-premium' ); ?></label>

			<select name='pdfemb[pdfemb_scrollbar]' id='pdfemb_scrollbar' class='select'>
				<option value="none" <?php echo $options['pdfemb_scrollbar'] === 'none' ? 'selected' : ''; ?>><?php esc_html_e( 'None', 'pdf-embedder-premium' ); ?></option>
				<option value="vertical" <?php echo $options['pdfemb_scrollbar'] === 'vertical' ? 'selected' : ''; ?>><?php esc_html_e( 'Vertical', 'pdf-embedder-premium' ); ?></option>
				<option value="horizontal" <?php echo $options['pdfemb_scrollbar'] === 'horizontal' ? 'selected' : ''; ?>><?php esc_html_e( 'Horizontal', 'pdf-embedder-premium' ); ?></option>
				<option value="both" <?php echo $options['pdfemb_scrollbar'] === 'both' ? 'selected' : ''; ?>><?php esc_html_e( 'Both', 'pdf-embedder-premium' ); ?></option>
			</select>

			<br/>

			<p class="desc clear">
				<?php esc_html_e( 'User can still use mouse if scrollbars are not visible.', 'pdf-embedder-premium' ); ?>
			</p>
		</div>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-page-continousscroll">
			<label for="pdfemb_continousscroll" class="textinput"><?php esc_html_e( 'Continuous Page Scrolling', 'pdf-embedder-premium' ); ?></label>

			<input type="checkbox" name='pdfemb[pdfemb_continousscroll]' id='pdfemb_continousscroll' class='checkbox' <?php echo Options::is_on( $options['pdfemb_continousscroll'] ) ? 'checked' : ''; ?> />
			<label for="pdfemb_continousscroll" class="checkbox plain">
				<?php esc_html_e( 'Allow user to scroll up/down between all pages in the PDF', 'pdf-embedder-premium' ); ?>
			</label>

			<p class="desc clear">
				<?php esc_html_e( 'If unchecked, user must click next/prev buttons on the toolbar to change pages.', 'pdf-embedder-premium' ); ?>
			</p>
		</div>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-page-download">
			<label for="pdfemb_download" class="textinput"><?php esc_html_e( 'Download Button', 'pdf-embedder-premium' ); ?></label>

			<input type="checkbox" name='pdfemb[pdfemb_download]' id='pdfemb_download' class='checkbox' <?php echo Options::is_on( $options['pdfemb_download'] ) ? 'checked' : ''; ?> />
			<label for="pdfemb_download" class="checkbox plain"><?php esc_html_e( 'Provide PDF download button in toolbar', 'pdf-embedder-premium' ); ?></label>
		</div>

		<br class="clear" />
		<br class="clear" />

		<div class="pdfemb-admin-setting pdfemb-admin-setting-page-tracking">
			<label for="pdfemb_tracking" class="textinput"><?php esc_html_e( 'Track Views/Downloads', 'pdf-embedder-premium' ); ?></label>

			<input type="checkbox" name='pdfemb[pdfemb_tracking]' id='pdfemb_tracking' class='checkbox' <?php echo Options::is_on( $options['pdfemb_tracking'] ) ? 'checked' : ''; ?> />
			<label for="pdfemb_tracking" class="checkbox plain">
				<?php esc_html_e( 'Count number of views and downloads', 'pdf-embedder-premium' ); ?>
			</label>
			<p class="desc clear">
				<?php
				printf(
					wp_kses( /* translators: %s: URL to Media Library filtered by PDF files. */
						__( 'Values will be shown in the <a href="%s" target="_blank">Media Library</a> for each individual file separately.', 'pdf-embedder-premium' ),
						[
							'a' => [
								'href'   => [],
								'target' => [],
							],
						]
					),
					esc_url( add_query_arg( 'attachment-filter', 'post_mime_type:application/pdf', admin_url( 'upload.php' ) ) )
				);
				?>
			</p>
		</div>

		<div class="pdfemb-admin-setting pdfemb-admin-setting-page-newwindow">
			<label for="pdfemb_newwindow" class="textinput"><?php esc_html_e( 'External Links', 'pdf-embedder-premium' ); ?></label>

			<input type="checkbox" name='pdfemb[pdfemb_newwindow]' id='pdfemb_newwindow' class='checkbox' <?php echo Options::is_on( $options['pdfemb_newwindow'] ) ? 'checked' : ''; ?> />
			<label for="pdfemb_newwindow" class="checkbox plain"><?php esc_html_e( 'Open links in a new browser tab/window', 'pdf-embedder-premium' ); ?></label>
		</div>

		<br class="clear" />
		<br class="clear" />

		<div class="pdfemb-admin-setting pdfemb-admin-setting-page-scrolltotop">
			<label for="pdfemb_scrolltotop" class="textinput"><?php esc_html_e( 'Scroll to Top', 'pdf-embedder-premium' ); ?></label>

			<input type="checkbox" name='pdfemb[pdfemb_scrolltotop]' id='pdfemb_scrolltotop' class='checkbox' <?php echo Options::is_on( $options['pdfemb_scrolltotop'] ) ? 'checked' : ''; ?> />
			<label for="pdfemb_scrolltotop" class="checkbox plain"><?php esc_html_e( 'Scroll to top of page when user clicks next/prev', 'pdf-embedder-premium' ); ?></label>
		</div>

		<br class="clear" />
		<br class="clear" />

		<div class="pdfemb-admin-setting pdfemb-admin-setting-page-search">
			<label for="pdfemb_search" class="textinput"><?php esc_html_e( 'Search Button', 'pdf-embedder-premium' ); ?></label>
			<input type="checkbox" name='pdfemb[pdfemb_search]' id='pdfemb_search' class='checkbox' <?php echo Options::is_on( $options['pdfemb_search'] ) ? 'checked' : ''; ?> />
			<label for="pdfemb_search" class="checkbox plain"><?php esc_html_e( 'Provides PDF search/find button in toolbar', 'pdf-embedder-premium' ); ?></label>
		</div>

		<br class="clear" />
		<br class="clear" />

		<?php
	}
}
