<?php

namespace PDFEmbedder\Premium\Plans\Features;

use PDFEmbedder\Options;

trait ExtendsOptionsWithSecure {

	/**
	 * Add the Secure options to the list of default values.
	 *
	 * @since 5.2.0
	 *
	 * @param array $defaults The default options.
	 */
	public function add_options_defaults( array $defaults ): array {

		return array_merge(
			$defaults,
			[
				'pdfemb_secure'            => true,
				'pdfemb_disablerightclick' => 'off',
				'pdfemb_secureattpages'    => false,
				'pdfemb_cacheencrypted'    => true,
				'pdfemb_wm_text'           => '',
				'pdfemb_wm_halign'         => 'center',
				'pdfemb_wm_voffset'        => '30',
				'pdfemb_wm_fontsize'       => '20',
				'pdfemb_wm_opacity'        => '20',
				'pdfemb_wm_rotate'         => '35',
				'pdfemb_wm_evenpages'      => false,
			]
		);
	}

	/**
	 * Validate the Premium options.
	 * The validation is done in steps according to each plan priority.
	 *
	 * @since 5.2.0
	 *
	 * @param array $validated Validated options, previously processed by other plans.
	 * @param array $input     Original options coming from the request.
	 */
	public function validate_options( array $validated, array $input ): array { // phpcs:ignore Generic.Metrics.CyclomaticComplexity

		$validated['pdfemb_secure']            = isset( $input['pdfemb_secure'] ) && Options::is_on( $input['pdfemb_secure'] );
		$validated['pdfemb_disablerightclick'] = isset( $input['pdfemb_disablerightclick'] ) && Options::is_on( $input['pdfemb_disablerightclick'] );

		$validated['pdfemb_cacheencrypted'] = isset( $input['pdfemb_cacheencrypted'] ) && Options::is_on( $input['pdfemb_cacheencrypted'] );

		$validated['pdfemb_secureattpages'] = isset( $input['pdfemb_secureattpages'] ) && Options::is_on( $input['pdfemb_secureattpages'] );

		$validated['pdfemb_wm_text'] = isset( $input['pdfemb_wm_text'] ) && ! preg_match( '/^[\n\r ]+$/', $input['pdfemb_wm_text'] ) ? $input['pdfemb_wm_text'] : '';

		if ( isset( $input['pdfemb_wm_halign'] ) && in_array( $input['pdfemb_wm_halign'], [ 'left', 'center', 'right' ], true ) ) {
			$validated['pdfemb_wm_halign'] = $input['pdfemb_wm_halign'];
		} else {
			$validated['pdfemb_wm_halign'] = 'center';
		}

		$validated['pdfemb_wm_voffset']  = isset( $input['pdfemb_wm_voffset'] ) ? (int) $input['pdfemb_wm_voffset'] : 5;
		$validated['pdfemb_wm_fontsize'] = isset( $input['pdfemb_wm_fontsize'] ) ? (int) $input['pdfemb_wm_fontsize'] : 20;
		$validated['pdfemb_wm_opacity']  = isset( $input['pdfemb_wm_opacity'] ) ? (int) $input['pdfemb_wm_opacity'] : 20;
		$validated['pdfemb_wm_rotate']   = isset( $input['pdfemb_wm_rotate'] ) ? (int) $input['pdfemb_wm_rotate'] : 35;

		$validated['pdfemb_wm_evenpages'] = isset( $input['pdfemb_wm_evenpages'] ) && Options::is_on( $input['pdfemb_wm_evenpages'] );

		return $validated;
	}
}
