<?php

namespace PDFEmbedder\Premium\Plans;

use ReflectionClass;

/**
 * Base class for all plans.
 *
 * @since 5.2.0
 */
abstract class PremiumPlan {

	/**
	 * Priority of the plan.
	 *
	 * @since 5.2.0
	 */
	const PRIORITY = 0;

	/**
	 * Name of the plan.
	 *
	 * @since 5.2.0
	 */
	const NAME = '';

	/**
	 * Initialize the plan.
	 *
	 * @since 5.2.0
	 */
	public function init() {
	}

	/**
	 * Extend the functionality of WP and PDF Lite.
	 * All these hooks may be running from various plans,
	 * which may or may not have methods implemented - that's why we have __call() method.
	 *
	 * @since 5.2.0
	 */
	public function hooks() {

		add_filter( 'pdfemb_options_defaults', [ $this, 'add_options_defaults' ], static::PRIORITY );
		add_filter( 'pdfemb_options_validated', [ $this, 'validate_options' ], static::PRIORITY, 2 );
	}

	/**
	 * Get the dynamic part of the plan FQCN to use in traits that are plan-agnostic.
	 *
	 * @since 5.2.0
	 */
	protected function get_namespace(): string {

		return ( new ReflectionClass( static::class ) )->getNamespaceName();
	}

	/**
	 * Intercept calls to undefined methods which means that the plan does not implement this feature.
	 *
	 * @since 5.2.0
	 *
     * @param string $method Name of the method.
     * @param array  $args   Arguments passed to the method.
	 */
	public function __call( string $method, array $args ) {

		$var = static function ( $params ) {
			return $params[0] ?? null;
		};

		return $var( $args );
	}
}
